﻿using System;
using System.Windows.Forms;

namespace ncvcommentmado
{
    public partial class CommentForm : Form
    {
        //
        // todo:    CommentForm フィールド
        //

        public delegate bool PostCommandEventHandler(
            string comment, int commentType, bool asAnonymous, 
            bool twitOrNot, string tweetconsolePath );
        public event PostCommandEventHandler PostCommand;

        const string SettingsDirName = "co1430045\\";
        const string SettingsFileName = "ncvcommentmado.commentform";
        const char SettingSplitter = ':';
        const string SettingNameXpos = "xpos";
        const string SettingNameYpos = "ypos";
        const string SettingNameWidth = "width";
        const string SettingNameHeight = "height";
        const string SettingNameTweetconsolepath = "tweetconsolepath";
        const string SettingNameTwitornotcheckbox = "twitornotcheckbox";
        const string SettingNameCommentTypeCombobox = "commenttypecombobox";
        const string SettingNameAsanonymouscheckbox = "asanonymouscheckbox";
        const string SettingNameIstopmostcheckbox = "istopmostcheckbox";

        //
        // todo:    CommentForm メソッド
        //

        public CommentForm()
        {
            InitializeComponent();
            selectTwitterClientButton.Tag = "";
            commentTypeComboBox.SelectedIndex = (int)CommentType.Owner;
        }

        public void LoadSettings( string settingsDirectoryPath )
        {
            commentTypeComboBox.SelectedIndex = (int)CommentType.Owner;
            string path = settingsDirectoryPath
                + SettingsDirName
                + SettingsFileName;
            string[] settings;

            try
            {
                settings = System.IO.File.ReadAllLines( path );
            }
            catch( System.IO.FileNotFoundException e )
            {
                return;
            }

            foreach ( string line in settings )
            {
                const int Name = 0;
                const int Value = 1;
                string[] setting = line.Split( SettingSplitter );

                switch( setting[ Name ] )
                {
                    case SettingNameXpos:
                        Left = Convert.ToInt32( setting[ Value ] );
                        break;
                    case SettingNameYpos:
                        Top = Convert.ToInt32( setting[ Value ] );
                        break;
                    case SettingNameWidth:
                        Width = Convert.ToInt32( setting[ Value ] );
                        break;
                    case SettingNameHeight:
                        Height = Convert.ToInt32( setting[ Value ] );
                        break;
                    case SettingNameTweetconsolepath:
                        if ( setting[ Value ] != "" )
                        {
                            selectTwitterClientButton.Tag = 
                                setting[ Value ] + ":" + setting[ Value + 1 ];
                            selectTwitterClientButton.Checked = true;
                        }
                        break;
                    case SettingNameTwitornotcheckbox:
                        twitOrNotCheckBox.Checked = 
                            Convert.ToBoolean( setting[ Value ] );
                        break;
                    case SettingNameCommentTypeCombobox:
                        commentTypeComboBox.SelectedIndex = 
                            Convert.ToInt32( setting[ Value ] );
                        break;
                    case SettingNameAsanonymouscheckbox:
                        asAnonymousCheckBox.Checked = 
                            Convert.ToBoolean( setting[ Value ] );
                        break;
                    case SettingNameIstopmostcheckbox:
                        isTopmostCheckBox.Checked = 
                            Convert.ToBoolean( setting[ Value ] );
                        break;
                }
            }
        }

        public void SaveSettings(string settingsDirectoryPath)
        {
            string settings = 
                SettingNameXpos + SettingSplitter + Location.X
                + "\n" + SettingNameYpos + SettingSplitter + Location.Y
                + "\n" + SettingNameWidth + SettingSplitter + Width
                + "\n" + SettingNameHeight + SettingSplitter + Height
                + "\n" + SettingNameTweetconsolepath + SettingSplitter 
                    + selectTwitterClientButton.Tag
                + "\n" + SettingNameTwitornotcheckbox + SettingSplitter 
                    + twitOrNotCheckBox.Checked
                + "\n" + SettingNameCommentTypeCombobox + SettingSplitter
                    + commentTypeComboBox.SelectedIndex
                + "\n" + SettingNameAsanonymouscheckbox + SettingSplitter 
                    + asAnonymousCheckBox.Checked
                + "\n" + SettingNameIstopmostcheckbox + SettingSplitter 
                    + isTopmostCheckBox.Checked;

            System.IO.Directory.CreateDirectory( 
                settingsDirectoryPath + SettingsDirName );
            System.IO.File.WriteAllText( 
                settingsDirectoryPath + SettingsDirName + SettingsFileName, 
                settings );
        }

        //
        // todo:    CommentForm プロパティ
        //

        //
        // todo:    CommentForm イベントハンドラ（的なもの）
        //

        // キーボードショートカット対応
        protected override bool ProcessCmdKey( ref Message msg, Keys keyData )
        {
            Keys code = keyData & Keys.KeyCode;
            Keys mod = keyData & Keys.Modifiers;

            if ( mod == Keys.Control )
            {
                switch ( code )
                {
                    case Keys.Enter:
                        post();
                        return true;
                    case Keys.A:
                        commentTextBox.SelectAll();
                        return true;
                }
            }
            return base.ProcessCmdKey( ref msg, keyData );
        }

        void postButton_Click( object sender, EventArgs e )
        {
            post();
            commentTextBox.Focus();
        }

        void isTopmostCheckBox_CheckedChanged( object sender, EventArgs e )
        {
            TopMost = isTopmostCheckBox.Checked;
        }

        void selectTwitterClientButon_Click( object sender, EventArgs e )
        {
            if ( selectTwitterClientButton.Checked )   // on
            {
                OpenFileDialog dlg = new OpenFileDialog();
                dlg.Title = "TweetConsoleを選択";
                if ( dlg.ShowDialog() == DialogResult.OK )
                    { selectTwitterClientButton.Tag = dlg.FileName; }
                else
                    { selectTwitterClientButton.Checked = false; }    
            }
            else
            {
                selectTwitterClientButton.Tag = "";
            }
        }

        public void HostBroadcastConnected()
        {
            postButton.Enabled = true;
        }

        public void HostBroadcastDisConnected()
        {
            postButton.Enabled = false;
        }

        //
        // todo:    CommentForm サブルーチン
        //

        void post()
        {
            if ( commentTextBox.Text.Length == 0 ) return;
            if ( !postButton.Enabled ) return;
            //if ( postButton.Enabled )
            //{
            // 入力欄の1行目のテキストを取得
            int index = commentTextBox.Text.IndexOf( "\r\n" );
            string textToPost;
                
            if ( index == -1 ) {    // 改行はなかった
                textToPost = commentTextBox.Text;
            }
            else {
                textToPost = commentTextBox.Text.Substring(0, index);
            }
                
            if ( PostCommand(
                    textToPost,
                    commentTypeComboBox.SelectedIndex,
                    asAnonymousCheckBox.Checked, 
                    twitOrNotCheckBox.Checked,
                    selectTwitterClientButton.Tag.ToString() ) )   // 成功
            {
                // 送信が成功したら、入力欄の1行目のみ削除する
                if ( index == -1 ) {    // 改行はなかった
                    commentTextBox.Text = "";
                }
                else {
                    commentTextBox.Text = commentTextBox.Text.Substring(
                        index + "\r\n".Length);
                }
            }
            //}
        }
    }
}
