﻿using System;
using System.Windows.Forms;//MessageBox.Show( "" );
using Plugin;
using NicoLibrary.NicoLiveAPI;

namespace ncvcommentmado
{
    enum CommentType { Owner, BSP, Listener };
    
    public class Main : IPlugin
    {
        //
        // todo:    Main フィールド
        //

        IPluginHost host = null;
        CommentForm commentForm = null;
        const string PluginName = "コメント窓";
        const string PluginVersion = "1.0";
        const string PluginDescription = "独立したコメント窓を表示する。";

        //
        // todo:    Main IPlugin メソッド
        //

        public void Run()
        {
            initializeCommentForm();

            host.BroadcastConnected += host_BroadcastConnected;
            host.BroadcastDisConnected += host_BroadcastDisConnected;
        }

        public void AutoRun()
        {
            Run();
        }

        //
        // todo:    Main IPlugin プロパティ
        //

        public bool IsAutoRun       { get { return true; } }
        public string Description   { get { return PluginDescription; } }
        public string Name          { get { return PluginName; } }
        public string Version       { get { return PluginVersion; } }
        public IPluginHost Host
        {
            get { return host; }
            set { host = value; }
        }

        //
        // todo:    Main イベントハンドラ
        //

        void host_BroadcastConnected( object sender, EventArgs e )
        {
            commentForm.HostBroadcastConnected();
        }

        void host_BroadcastDisConnected( object sender, EventArgs e )
        {
            commentForm.HostBroadcastDisConnected();
        }

        void commentForm_FormClosed( object sender, FormClosedEventArgs e )
        {
            uninitializeCommentForm();

            host.BroadcastConnected -= host_BroadcastConnected;
            host.BroadcastDisConnected -= host_BroadcastDisConnected;
        }

        //
        // todo:    Main デリゲート
        //

        bool commentForm_PostCommand( 
            string comment, int commentType, bool asAnonymous, 
            bool twitOrNot, string tweetconsolePath )
        {
            bool result;

            if ( commentType == (int)CommentType.Owner )  // 運コメ
            {
                try
                {   // 放送主でなければ必ず失敗する
                    result = host.SendOwnerComment( 
                        asAnonymous, "", "", "", comment, "", "" );
                }
                catch
                {
                    result = host.SendComment( 
                        asAnonymous, "", "", "", comment );
                    commentType = (int)CommentType.Listener;
                }
            }
            else if ( commentType == (int)CommentType.BSP ) // BSP
            {
                try
                {   // BSPでなければたぶん失敗する
                    result = host.SendPresscastComment ( 
                        "", comment, "" );
                }
                catch
                {
                    result = host.SendComment( 
                        asAnonymous, "", "", "", comment );
                    commentType = (int)CommentType.Listener;
                }
            }
            else
            {
                result = host.SendComment( 
                    asAnonymous, "", "", "", comment );
            }

            if ( !result ) { return false; }

            if ( twitOrNot && tweetconsolePath != "" )
            {
                string args;
                NicoLivePlayerStatus playerStatus = host.GetPlayerStatus();
                
                args = "/t " + "\"" + "[ニコ生※] " + comment;
                if ( commentType == (int)CommentType.Owner ) 
                    { args = args + " [放送中] "; }
                else 
                    { args = args + " [視聴中] "; }
                args = args + "http://live.nicovideo.jp/watch/lv"
                    + playerStatus.LiveNum + "\"";

                System.Diagnostics.Process.Start(
                    tweetconsolePath, args );
            }

            return true;
        }

        //
        // todo:    Main サブルーチン
        //

        void initializeCommentForm()
        {
            commentForm = new CommentForm();
            commentForm.Text = PluginName;

            commentForm.PostCommand += commentForm_PostCommand;
            commentForm.FormClosed += commentForm_FormClosed;

            commentForm.Show( host.MainForm );
            commentForm.LoadSettings( host.DirectoryPathSettingFile );
        }

        void uninitializeCommentForm()
        {
            commentForm.SaveSettings( host.DirectoryPathSettingFile );
            commentForm.PostCommand -= commentForm_PostCommand;
            commentForm.FormClosed -= commentForm_FormClosed;
        }
    }
}
